package internal

import (
	"flag"
	"runtime"
	"time"

	"github.com/open-io/oio-exporter/common"
	"github.com/open-io/oio-exporter/daemon"
)

var confInterval = time.Duration(0)

func InitFlags(fs *flag.FlagSet) {
	fs.DurationVar(&confInterval, "internal.interval", confInterval, "[internal] interval between polls")
}

type internalDaemon struct {
	daemon.Daemon

	memAllocMax uint64
}

func MakeDaemon() *internalDaemon {
	return &internalDaemon{
		memAllocMax: 0,
	}
}

func (d *internalDaemon) ID() string {
	return "Internal"
}

func (d *internalDaemon) Interval() time.Duration {
	return confInterval
}

func (d *internalDaemon) Collect(m *common.MetricData) error {
	var mem runtime.MemStats
	runtime.ReadMemStats(&mem)
	if mem.Alloc > d.memAllocMax {
		d.memAllocMax = mem.Alloc
	}
	m.Set(&common.Metric{
		Name:   "self_mem_alloc_current",
		Labels: [][]string{},
		Value:  float64(mem.Alloc),
	})
	m.Set(&common.Metric{
		Name:   "self_mem_alloc_max",
		Labels: [][]string{},
		Value:  float64(d.memAllocMax),
	})
	m.Set(&common.Metric{
		Name:   "self_num_goroutines",
		Labels: [][]string{},
		Value:  float64(runtime.NumGoroutine()),
	})

	return nil
}
